package tech.mercantec.easyeat.ui.dishes

import android.app.Activity
import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.PopupMenu
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.Fragment
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import tech.mercantec.easyeat.R
import tech.mercantec.easyeat.databinding.FragmentDishesBinding
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.GetAllRecipesResponse
import tech.mercantec.easyeat.helpers.deleteRecipe
import tech.mercantec.easyeat.helpers.getAllRecipes
import tech.mercantec.easyeat.helpers.getRecipeDetails
import tech.mercantec.easyeat.models.DishListItem
import tech.mercantec.easyeat.models.Recipe
import kotlin.concurrent.thread

class DishesFragment : Fragment() {

    private var _binding: FragmentDishesBinding? = null
    private lateinit var launcher: ActivityResultLauncher<Intent>
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentDishesBinding.inflate(inflater, container, false)
        val root: View = binding.root

        loadRecipes()

        // Show details on click
        binding.dishesList.setOnItemClickListener { parent, view, position, id ->
            val item = parent.getItemAtPosition(position) as DishListItem

            val intent = Intent(requireContext(), DishDetailsActivity::class.java)
            intent.putExtra("dish_id", item.id)
            startActivity(intent)
        }

        // Show context menu when long clicking shopping item
        binding.dishesList.setOnItemLongClickListener { parent, view, position, id ->
            val item = parent.getItemAtPosition(position) as DishListItem

            val popup = PopupMenu(requireActivity(), view)
            popup.apply {
                menuInflater.inflate(R.menu.dish_context_menu, menu)

                setOnMenuItemClickListener {
                    when (it.itemId) {
                        // Open edit dish activity when editing
                        R.id.edit_dish -> {
                            thread {
                                // Get full recipe details
                                val recipe: Recipe
                                try {
                                    recipe = getRecipeDetails(requireContext(), item.id)
                                } catch (e: ApiRequestException) {
                                    activity?.runOnUiThread {
                                        Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                                    }

                                    return@thread
                                }

                                val intent = Intent(activity, EditDishActivity::class.java)
                                intent.putExtra("recipe_json", Json.encodeToString(recipe))
                                launcher.launch(intent)
                            }

                            true
                        }
                        R.id.remove_dish -> {
                            AlertDialog.Builder(activity)
                                .setTitle("Delete dish")
                                .setMessage("Are you sure you want to delete this dish?")
                                .setPositiveButton(R.string.delete_label) { _, _ ->
                                    (parent.adapter as DishAdapter).remove(item)

                                    thread {
                                        try {
                                            deleteRecipe(requireContext(), item.id)
                                        } catch (e: ApiRequestException) {
                                            activity?.runOnUiThread {
                                                Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                                            }
                                        }
                                    }
                                }
                                .setNegativeButton(R.string.cancel_label, null)
                                .show()

                            true
                        }
                        else -> false
                    }
                }

                show()
            }

            return@setOnItemLongClickListener true
        }

        binding.addDish.setOnClickListener {
            val popup = PopupMenu(requireActivity(), it)

            popup.apply {
                menuInflater.inflate(R.menu.create_dish_menu, menu)

                setOnMenuItemClickListener {
                    when (it.itemId) {
                        R.id.create_manually -> {
                            val intent = Intent(requireContext(), CreateDishActivity::class.java)
                            launcher.launch(intent)

                            true
                        }
                        R.id.create_with_ai -> {
                            val intent = Intent(requireContext(), GenerateRecipeActivity::class.java)
                            launcher.launch(intent)

                            true
                        }
                        else -> false
                    }
                }

                show()
            }
        }


        return root

    }

    fun DishesFragment.loadRecipes() {
        binding.dishesList.visibility = View.GONE
        binding.emptyDishesList.visibility = View.GONE
        binding.addDish.visibility = View.GONE
        binding.loading.visibility = View.VISIBLE

        thread {
            val recipes: List<GetAllRecipesResponse>
            try {
                recipes = getAllRecipes(requireContext())
            } catch (e: ApiRequestException) {
                activity?.runOnUiThread {
                    Toast.makeText(requireContext(), e.message, Toast.LENGTH_LONG).show()
                }
                return@thread
            }

            val dishes = recipes.map {
                DishListItem(it.id, it.name, it.description)
            }

            activity?.runOnUiThread {
                binding.dishesList.visibility = View.VISIBLE
                binding.emptyDishesList.visibility = View.VISIBLE
                binding.addDish.visibility = View.VISIBLE
                binding.loading.visibility = View.GONE

                binding.dishesList.emptyView = binding.emptyDishesList

                val adapter = DishAdapter(requireContext(), dishes)
                binding.dishesList.adapter = adapter
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        launcher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                loadRecipes()
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
